/**
 * @file analyzer.d
 * @brief A GUI model viewer
 * @author Sebastien Alaiwan
 * @date 2014-10-28
 */

/*
 * Copyright (C) 2014 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

import std.stdio;
import std.string;
import model;
import xpm;

import glib.Str;
import gdk.Pixbuf;

import gtk.CellRendererPixbuf;
import gtk.CellRendererText;
import gtk.Image;
import gtk.Label;
import gtk.Main;
import gtk.MainWindow;
import gtk.ScrolledWindow;
import gtk.TreeIter;
import gtk.TreePath;
import gtk.TreeSelection;
import gtk.TreeStore;
import gtk.TreeView;
import gtk.TreeViewColumn;
import gtk.VBox;

void runAnalyzer(in MODEL m, string[] args)
{
  Main.init(args);
  auto mainWnd = new AnalyzerMainWindow(m);
  Main.run();
}

class AnalyzerMainWindow : MainWindow
{
public:
  this(in MODEL m)
  {
    super("Analyzer");
    add(new AnalyzerTreeView(m));
    resize(400, 600);
    showAll();
  }
}

class AnalyzerTreeView : VBox
{
public:
  this(in MODEL m)
  {
    super(false, 0);

    TreeView treeView1 = makeTreeView(m);

    auto sw = new ScrolledWindow(null, null);
    sw.add(treeView1);
    packStart(sw, true, true, 1);
  }

private:
  TreeView makeTreeView(in MODEL m)
  {
    class TTreeStore : TreeStore
    {
      this()
      {
        GType[] columns;
        columns ~= Pixbuf.getType();
        columns ~= GType.STRING;
        columns ~= GType.STRING;
        super(columns);
      }
    }

    auto store = new TTreeStore();
    TreeView treeView = new TreeView(store);
    treeView.setRulesHint(true);

    TreeViewColumn column;

    int col = 0;
    column =
      new TreeViewColumn("Icon", new CellRendererPixbuf(), "pixbuf", col);
    treeView.appendColumn(column);
    ++col;

    column = new TreeViewColumn("Name", new CellRendererText(), "text", col);
    treeView.appendColumn(column);
    column.setResizable(true);
    ++col;

    column = new TreeViewColumn("Type", new CellRendererText(), "text", col);
    treeView.appendColumn(column);
    column.setResizable(true);
    ++col;

    populate(store, m);

    return treeView;
  }
}

void populate(TreeStore store, in MODEL m)
{
  auto i = store.createIter();

  foreach(block; m)
  {
    populateBlock(store, i, block);
  }
}

void populateBlock(TreeStore store, TreeIter i, const Block block)
{
  void doSequence(const SequenceBlock header)
  {
    foreach(stmt; header.statements)
    {
      auto j = store.append(i);
      populateBlock(store, j, stmt);
    }
  }

  void doNamedSequence(const NamedSequenceBlock header)
  {
    store.setValue(i, 0, new Pixbuf(package_xpm));
    store.setValue(i, 1, header.sName);

    doSequence(header.seq);
  }

  void doField(const FieldBlock member)
  {
    store.setValue(i, 0, new Pixbuf(greenInterface_xpm));
    store.setValue(i, 1, member.sName);
    store.setValue(i, 2, member.sType);
  }

  void doIf(const IfBlock ifBlock)
  {
    store.setValue(i, 0, new Pixbuf(greenInterface_xpm));
    store.setValue(i, 1, "if(" ~ ifBlock.condition ~ ")");
    store.setValue(i, 2, "(condition)");

    auto j1 = store.append(i);
    store.setValue(j1, 1, "then");
    populateBlock(store, j1, ifBlock.m_then);
    auto j2 = store.append(i);
    store.setValue(j2, 1, "else");
    populateBlock(store, j2, ifBlock.m_else);
  }

  block.visit(
    &doNamedSequence,
    &doSequence,
    &doIf,
    &doField
    );
}

