/**
 * @file model.d
 * @author Sebastien Alaiwan
 * @date 2014-10-08
 */

/*
 * Copyright (C) 2014 - Sebastien Alaiwan
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

alias Block[] MODEL;

class Block
{
public:
  this(NamedSequenceBlock sb)
  {
    m_type = Type.NamedSequence;
    m_namedSequence = sb;
  }

  this(SequenceBlock sb)
  {
    m_type = Type.Sequence;
    m_sequence = sb;
  }

  this(FieldBlock fb)
  {
    m_type = Type.Field;
    m_field = fb;
  }

  this(IfBlock ib)
  {
    m_type = Type.If;
    m_if = ib;
  }

  void visit(void delegate(const NamedSequenceBlock) visitNamedSequence,
             void delegate(const SequenceBlock) visitSequence,
             void delegate(const IfBlock) visitIf,
             void delegate(const FieldBlock) visitField) const
  {
    final switch(m_type)
    {
    case Type.NamedSequence:
      visitNamedSequence(m_namedSequence);
      break;
    case Type.Sequence:
      visitSequence(m_sequence);
      break;
    case Type.Field:
      visitField(m_field);
      break;
    case Type.If:
      visitIf(m_if);
      break;
    }
  }

  const(FieldBlock) asField() const
  {
    assert(m_type == Type.Field);
    return m_field;
  }

  const(NamedSequenceBlock) asNamedSequence() const
  {
    assert(m_type == Type.NamedSequence);
    return m_namedSequence;
  }

private:
  enum Type
  {
    NamedSequence,
    Sequence,
    Field,
    If,
  }

  Type m_type;

  union
  {
    NamedSequenceBlock m_namedSequence;
    SequenceBlock m_sequence;
    FieldBlock m_field;
    IfBlock m_if;
  }
}

class NamedSequenceBlock
{
  string sName;
  SequenceBlock seq;
};

class SequenceBlock
{
  Block[] statements;
};

class IfBlock
{
  string condition;
  Block m_then;
  Block m_else;
};

class FieldBlock
{
  int iSizeInBytes;
  string sType;
  string sName;
  int iArrayLength;
  string sComment;
};

Block mkField(int iSizeInBytes, string sType, string sName, int iArrayLength,
              string sComment)
{
  auto r = new FieldBlock;
  r.iSizeInBytes = iSizeInBytes;
  r.sType = sType;
  r.sName = sName;
  r.iArrayLength = iArrayLength;
  r.sComment = sComment;
  return new Block(r);
}

Block mkNamedSequence(string name, Block[] fields)
{
  auto r = new NamedSequenceBlock;
  r.sName = name;
  r.seq = new SequenceBlock;
  r.seq.statements = fields;
  return new Block(r);
}

Block mkSequence(Block[] fields)
{
  auto r = new SequenceBlock;
  r.statements = fields;
  return new Block(r);
}

Block mkIf(string cond, Block thenBlock, Block elseBlock)
{
  auto r = new IfBlock;
  r.condition = cond;
  r.m_then = thenBlock;
  r.m_else = elseBlock;
  return new Block(r);
}

